/******************************************************************************
* To the extent possible under law, Sebastian Roith
* <https://www.roith-unterrichtet.de> has waived all copyright and related or
* neighboring rights to this script. This work is published from Germany.
******************************************************************************/

/**
* Klasse für einfache Programmieraufgaben mit mehreren Teilaufgaben
*/
function RoboterCodingLmsConnector(
    node        // DOM-Knoten, der für die Benutzeroberfläche genutzt werden soll
) {
    this.upgradeLicenceBtn();
    this._ui = new RoboterCodingTaskUI(node);
    this.connect();
    this._ui.onlevelunlock = (() => this.saveScore());
    this._ui.onnewfine = (() => this.saveScore());
    this._ui.oncodeChange = ((code) => this._procCodeChange(code));
}

// Genutzte RoboterCodingTaskUI-Instanz
RoboterCodingLmsConnector.prototype._ui = null;

/**
* Schaltfläche zum Einblenden der Lizenz-Infos anzeigen
* @return undefined
*/
RoboterCodingLmsConnector.prototype.upgradeLicenceBtn = function() {
    new ScoTools.InfoPageLink();
}

/**
* Zufallszahl generieren, die als festes Start-Seed für die Pseudozufallszahlengenerierung verwendet wird
* @return Number    Seed
*/
RoboterCodingLmsConnector.genSeed = function() {
    return Math.floor(Math.random() * 233280);
}

/**
* Versuchen, eine Verbindung zur Lernplattform aufzubauen
* @return undefined
*/
RoboterCodingLmsConnector.prototype.connect = function() {
    if (typeof ScoTools.LMSLink == 'function') {
        var self = this;
        this.lmslink = new ScoTools.LMSLink();
        this.lmslink.onnew = (function() {
            self.lmsValuesInit();
        });
        this.lmslink.start();
        // Werte laden
        try {
            const lessonLocation = this.lmslink.lessonLocation;
            const data = JSON.parse('[' + lessonLocation + ']');
            if (data.length < 1)
                data.push(0);
            if (data.length < 2)
                data.push(0);
            if (data.length < 3)
                data.push(RoboterCodingLmsConnector.genSeed());
            this._ui.loadValues(+data[0], +data[1], +data[2], this.lmslink.suspendData);
            const newDataStr = data.join(',');
            if (newDataStr != lessonLocation)
                this.lmslink.lessonLocation = newDataStr;
        } catch(e) {
            const data =[0, 0, RoboterCodingLmsConnector.genSeed()];
            if (this.canSave())
                this._ui.loadValues(data[0], data[1], data[2]);
            this.lmslink.lessonLocation = data.join(',');
        }
    }
    if (!this.lmslink.ready)
        this._ui.setGeneralMsg('Verbindungsaufnahme zur Lernplattform fehlgeschlagen - Fortschritte können nicht gespeichert werden', 'msg_warn');
}

/**
* Beim ersten Start des Lernpaketes Anfangswerte festlegen
* @return undefined
*/
RoboterCodingLmsConnector.prototype.lmsValuesInit = function() {
    this.lmslink.lessonLocation = '0,0,' + RoboterCodingLmsConnector.genSeed();
}

// Genutzte Instanz von LMSLink
RoboterCodingLmsConnector.prototype.lmslink = { 'ready': false, lessonLocation: '0,0,' + RoboterCodingLmsConnector.genSeed() };

/**
* Ermittelt, ob die Aufgabe bereits erfolgreich gelöst wurde
* @return Boolean       Aufgabe bereits gelöst?
*/
RoboterCodingLmsConnector.prototype.getDone = function() {
    return (this.lmslink.ready && !this.lmslink.isIncomplete());
}

/**
* Prüft, ob Veränderungen am Lernpaket gespeichert werden sollen
* @return Boolean       Kann und soll gespeichert werden?
*/
RoboterCodingLmsConnector.prototype.canSave = function() {
    return (!this.lmslink.ready || this.lmslink.isNormalIncomplete());
}

/**
* Bearbeitungszeit speichern
* @return undefined
*/
RoboterCodingLmsConnector.prototype.reportTime = function() {
    this.lmslink.ready && this.lmslink.reportTime();
}

/**
* Punktestand und Status speichern
* @return undefined
*/
RoboterCodingLmsConnector.prototype.saveScore = function(
    scoreRaw,       // Erreichter Punktestand
    scoreMin,       // Minimal erreichbarer Punktestand
    scoreMax,       // Maximal erreichbarer Punktestand
    status          // Optional: lessonStatus
) {
    if (!this.canSave()) {
        this.reportTime();
        return;
    }
    const data = { lessonLocation: this._ui.unlockedLevelCount + ',' + this._ui.fineCount + ',' + this._ui.startSeed };
    if (this._ui.done) {
        data.lessonStatus = 'completed';
        data.scoreMin = scoreMin;
        data.scoreMax = scoreMax;
        data.score = scoreRaw;
    }
    if (this._saveCodeTimeout !== false) {
        clearTimeout(this._saveCodeTimeout);
        this._saveCodeTimeout = false;
        data.suspendData = this._saveCodeContent;
    }
    this.save(data);
}

/**
* Einen Wert vom LMS lesen
* @return undefined
*/
RoboterCodingLmsConnector.prototype.read = function(
    key     // Attributname von LMSLink-Instanz, die gelesen werden soll
) {
    return (this.lmslink.ready ? this.lmslink[key] : undefined);
}

/**
* Einen Wert vom LMS speichern lassen
* @return undefined
*/
RoboterCodingLmsConnector.prototype.write = function(
    key,        // Attributname von LMSLink-Instanz, unter dem der Wert gespeichert werden soll
    value       // Zu speichernder Wert
) {
    if (this.lmslink.ready)
        this.lmslink[key] = value;
}

/**
* Zugleich mehrere Werte ändern, die Zeit speichern und dann erst Commit an das LMS senden
* @return undefined
*/
RoboterCodingLmsConnector.prototype.save = function(
    data        // Objekt mit zu speichernden Daten
) {
    if (this.lmslink.ready)
        this.lmslink.save(data);
}

/**
* Quellcode sichern
* @return undefined
*/
RoboterCodingLmsConnector.prototype.saveCode = function(
    code        // Zu speichernder Quellcode
) {
    if (!this.canSave())
        return;
    this.save({ suspendData: code });
}

RoboterCodingLmsConnector.prototype._saveCodeTimeout = false;
RoboterCodingLmsConnector.prototype._saveCodeContent = '';

/**
* Eingegebenen Quellcode zeitversetzt speichern
* @return undefined
*/
RoboterCodingLmsConnector.prototype._procCodeChange = function(
    code        // Zu speichernder Quellcode
) {
    if (code.length > 2048)
        return;
    this._saveCodeContent = code;
    if (this._saveCodeTimeout === false)
        this._saveCodeTimeout = setTimeout(() => { this._saveCodeTimeout = false; this.saveCode(this._saveCodeContent); }, 1000);
}
